"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pluginFunction = void 0;
const tslib_1 = require("tslib");
const ts = (0, tslib_1.__importStar)(require("typescript"));
const path_1 = require("path");
const typescript_transformer_common_1 = require("@build-script/typescript-transformer-common");
const node_1 = require("@idlebox/node");
const fs_extra_1 = require("fs-extra");
/** @external */
exports.pluginFunction = (0, typescript_transformer_common_1.createProgramPlugin)(function plugin(program, { verbose } = {}, { rootDir, outDir, packageJsonPath }) {
    const debug = (0, typescript_transformer_common_1.getDebug)(verbose || false);
    debug('new ts.Program... %s files', program.getSourceFiles().length);
    const wrapImportFileName = function (f) {
        return f.replace(rootDir, outDir).replace(/\.tsx?$/, '.js') + '.importinfo';
    };
    debug('load package.json: %s', packageJsonPath);
    const dependencies = (0, typescript_transformer_common_1.createDependencies)(packageJsonPath);
    debug('    dependencies count: %s', Object.keys(dependencies).length);
    const typeChecker = program.getTypeChecker();
    return function transformer(transformationContext) {
        transformationContext.enableEmitNotification(ts.SyntaxKind.SourceFile);
        return (sourceFile) => {
            var _a, _b;
            const { fileName } = sourceFile;
            const importInfoFile = { imports: [], errors: [] };
            debug('[trans] visit \x1B[2m%s\x1B[0m', fileName);
            const imports = (0, typescript_transformer_common_1.getAllImports)(sourceFile);
            const importsWithoutBuiltin = imports.filter((importStat) => {
                const importId = (0, typescript_transformer_common_1.getImportedName)(importStat);
                return !(0, typescript_transformer_common_1.isImportNodeBuiltins)(importId);
            });
            const importInfoResult = (0, typescript_transformer_common_1.collectImportInfo)(sourceFile, importsWithoutBuiltin, typeChecker);
            for (const importStat of imports) {
                const importId = (0, typescript_transformer_common_1.getImportedName)(importStat);
                if ((0, typescript_transformer_common_1.isImportNodeBuiltins)(importId)) {
                    continue;
                }
                const info = (0, typescript_transformer_common_1.isImportFromNodeModules)(importStat)
                    ? (0, typescript_transformer_common_1.resolveTypescriptModule)(importStat, packageJsonPath)
                    : (0, typescript_transformer_common_1.resolveProjectFile)(importStat, program);
                if (info.type === 'missing') {
                    importInfoFile.errors.push(info);
                    continue;
                }
                const importInfo = importInfoResult.get(importStat);
                importInfoFile.imports.push({
                    ...info,
                    values: (_a = importInfo === null || importInfo === void 0 ? void 0 : importInfo.values) !== null && _a !== void 0 ? _a : [],
                    types: (_b = importInfo === null || importInfo === void 0 ? void 0 : importInfo.types) !== null && _b !== void 0 ? _b : [],
                });
            }
            debug('[trans] visit complete');
            transformationContext.onEmitNode(ts.EmitHint.SourceFile, sourceFile, () => {
                const distFile = wrapImportFileName(fileName);
                // console.log('---------------', distFile, importInfoFile);
                (0, fs_extra_1.mkdirpSync)((0, path_1.dirname)(distFile));
                if (importInfoFile.errors.length === 0) {
                    delete importInfoFile.errors;
                }
                (0, node_1.writeFileIfChangeSync)(distFile, JSON.stringify(importInfoFile, null, 4));
            });
            return sourceFile;
        };
    };
});
//# sourceMappingURL=transform.js.map